# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create a new certificate template."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.privateca import request_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.privateca import flags_v1
from googlecloudsdk.command_lib.privateca import operations
from googlecloudsdk.command_lib.privateca import resource_args
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  r"""Create a new certificate template."""

  detailed_help = {
      'DESCRIPTION':
          """\
          Create a certificate template that enforces policy restrictions on
          certificate requestors. Using a certificate template, you can define
          restrictions on the kinds of Subjects/SANs and x509 extensions allowed
          from certificate requestors as well as a default set of x509
          extensions that should be applied to all certificates using that
          template. These templates can be binded to IAM identities such that
          certain groups of requestors must use particular templates, allowing
          for fine-grained policy enforcements based on identity.

          For more information and examples, see https://cloud.google.com/certificate-authority-service/docs/creating-certificate-template.
          """,
      'EXAMPLES':
          """\
        To create a template that prohibits any x509 extension from a requester,
        but permits custom subjects/SANs and defines the default x509
        extensions, run:

          $ {command} restricted-template --location=us-west1 --copy-subject --copy-sans --predefined-values-file=x509_parameters.yaml

        To create a template that allows requesters to specify only DNS names
        from requesters, use a custom CEL expression with a SAN only restriction:

          $ {command} dns-only-template --location=us-west1 --description="Restricts certificates to DNS SANs." --no-copy-subject --copy-sans --identity-cel-expression="subject_alt_names.all(san, san.type == DNS)"

        To create a template that permits a requestor to specify extensions by
        OIDs, and subjects (but not SANs), with default x509 exensions:

          $ {command} mtls-only-extensions --location=us-west1 --copy-subject --no-copy-sans --predefined-values-file=mtls_cert_exts.yaml --copy-extensions-by-oid=1.3.6.1.5.5.7.3.2,1.3.6.1.5.5.7.3.1
       """,
  }

  @staticmethod
  def Args(parser):
    resource_args.AddCertificateTemplatePositionalResourceArg(parser,
                                                              'to create')
    base.Argument(
        '--description',
        help='A text description for the Certificate Template.').AddToParser(
            parser)
    flags_v1.AddPredefinedValuesFileFlag(parser)
    flags_v1.AddIdentityConstraintsFlags(parser)
    flags_v1.AddExtensionConstraintsFlags(parser)
    flags_v1.AddMaximumLifetimeFlag(parser)
    labels_util.AddCreateLabelsFlags(parser)

  def Run(self, args):
    client = privateca_base.GetClientInstance('v1')
    messages = privateca_base.GetMessagesModule('v1')

    cert_template_ref = args.CONCEPTS.certificate_template.Parse()

    flags_v1.ValidateIdentityConstraints(args)

    new_cert_template = messages.CertificateTemplate(
        predefinedValues=flags_v1.ParsePredefinedValues(args),
        identityConstraints=flags_v1.ParseIdentityConstraints(args),
        passthroughExtensions=flags_v1.ParseExtensionConstraints(args),
        description=args.description
        if args.IsSpecified('description')
        else None,
        maximumLifetime=flags_v1.ParseMaximumLifetime(args),
    )

    operation = client.projects_locations_certificateTemplates.Create(
        messages.PrivatecaProjectsLocationsCertificateTemplatesCreateRequest(
            parent=cert_template_ref.Parent().RelativeName(),
            certificateTemplateId=cert_template_ref.Name(),
            certificateTemplate=new_cert_template,
            requestId=request_utils.GenerateRequestId()))

    cert_template_response = operations.Await(
        operation, 'Creating Certificate Template.', api_version='v1')
    cert_template = operations.GetMessageFromResponse(
        cert_template_response, messages.CertificateTemplate)

    log.status.Print('Created Certificate Template [{}].'.format(
        cert_template.name))



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The gcloud datastore indexes cleanup command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.app import yaml_parsing
from googlecloudsdk.api_lib.datastore import index_api
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.app import output_helpers
from googlecloudsdk.command_lib.datastore import flags
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io


class Cleanup(base.Command):
  """Cleanup Cloud Datastore indexes."""

  detailed_help = {
      'brief':
          'Remove unused datastore indexes based on your local index '
          'configuration.',
      'DESCRIPTION':
          """
This command removes unused datastore indexes based on your local index
configuration. Any indexes that exist that are not in the index file will be
removed.
      """,
      'EXAMPLES':
          """\
          To remove unused indexes based on your local configuration, run:

            $ {command} ~/myapp/index.yaml
          """,
  }

  @staticmethod
  def Args(parser):
    """Get arguments for this command.

    Args:
      parser: argparse.ArgumentParser, the parser for this command.
    """
    flags.AddIndexFileFlag(parser)

  def Run(self, args):
    project = properties.VALUES.core.project.Get(required=True)
    info = yaml_parsing.ConfigYamlInfo.FromFile(args.index_file)
    if not info or info.name != yaml_parsing.ConfigYamlInfo.INDEX:
      raise exceptions.InvalidArgumentException(
          'index_file', 'You must provide the path to a valid index.yaml file.')
    output_helpers.DisplayProposedConfigDeployments(
        project=project, configs=[info]
    )
    console_io.PromptContinue(
        default=True, throw_if_unattended=False, cancel_on_no=True)
    received_indexes = index_api.NormalizeIndexes(info.parsed.indexes or [])
    indexes_to_delete_ids = set()
    current_indexes = index_api.ListIndexes(project)
    for index_id, index in current_indexes:
      if index in received_indexes:
        continue
      msg = ('This index is no longer defined in your index.yaml file.\n{0}'
             .format(index.ToYAML()))
      prompt = 'Do you want to delete this index'
      if console_io.PromptContinue(msg, prompt, default=True):
        indexes_to_delete_ids.add(index_id)
    index_api.DeleteIndexes(project, indexes_to_delete_ids)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Factory for JupyterConfig message."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.util.apis import arg_utils


class JupyterConfigFactory(object):
  """Factory for JupyterConfig message.

  Factory to add JupyterConfig message arguments to argument parser and create
  JupyterConfig message from parsed arguments.
  """

  def __init__(self, dataproc):
    """Factory for JupyterConfig message.

    Args:
      dataproc: A api_lib.dataproc.Dataproc instance.
    """
    self.dataproc = dataproc

  def GetMessage(self, args):
    """Builds a JupyterConfig message according to user settings.

    Args:
      args: Parsed arguments.

    Returns:
      JupyterConfig: A JupyterConfig message instance.
    """
    jupyter_config = self.dataproc.messages.JupyterConfig()
    if args.kernel:
      jupyter_config.kernel = arg_utils.ChoiceToEnum(
          args.kernel,
          self.dataproc.messages.JupyterConfig.KernelValueValuesEnum,
      )

    return jupyter_config


def AddArguments(parser):
  """Adds arguments related to JupyterConfig message to the given parser."""
  parser.add_argument(
      '--kernel',
      choices=['python', 'scala'],
      help=('Jupyter kernel type. The value could be "python" or "scala".'))



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Remote repo utils for Artifact Registry repository commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.artifacts import exceptions as ar_exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.artifacts import requests as ar_requests
from googlecloudsdk.command_lib.util.apis import arg_utils


def Args():
  """Adds the remote-<facade>-repo flags."""
  # We need to do this because these flags need to be able to accept either a
  # PublicRepository enum or a string registry URI.
  hide_custom_remotes = True
  return [
      base.Argument(
          "--remote-mvn-repo",
          help=_RemoteRepoHelpText("Maven", hide_custom_remotes),
      ),
      base.Argument(
          "--remote-docker-repo",
          help=_RemoteRepoHelpText("Docker", hide_custom_remotes),
      ),
      base.Argument(
          "--remote-npm-repo",
          help=_RemoteRepoHelpText("Npm", hide_custom_remotes),
      ),
      base.Argument(
          "--remote-python-repo",
          help=_RemoteRepoHelpText("Python", hide_custom_remotes),
      ),
      base.Argument(
          "--remote-apt-repo",
          help=_OsPackageRemoteRepoHelpText("Apt", hide_custom_remotes),
      ),
      base.Argument(
          "--remote-yum-repo",
          help=_OsPackageRemoteRepoHelpText(
              "Yum",
              hide_custom_remotes,
          ),
      ),
      base.Argument(
          "--remote-username",
          help="Remote Repository upstream registry username.",
          hidden=True,
      ),
      base.Argument(
          "--remote-password-secret-version",
          help="""\
          Secret Manager secret version that contains password for the
          remote repository upstream.
          """,
          hidden=True,
      ),
  ]


def IsRemoteRepoRequest(repo_args) -> bool:
  """Returns whether or not the repo mode specifies a remote repository."""
  return (
      hasattr(repo_args, "mode")
      and arg_utils.ChoiceToEnumName(repo_args.mode) == "REMOTE_REPOSITORY"
  )


def AppendRemoteRepoConfigToRequest(messages, repo_args, request):
  """Adds remote repository config to CreateRepositoryRequest or UpdateRepositoryRequest."""
  remote_cfg = messages.RemoteRepositoryConfig()
  remote_cfg.description = repo_args.remote_repo_config_desc
  # Credentials
  username = repo_args.remote_username
  secret = repo_args.remote_password_secret_version
  if username or secret:
    creds = messages.UpstreamCredentials()
    creds.usernamePasswordCredentials = messages.UsernamePasswordCredentials()
    if username:
      creds.usernamePasswordCredentials.username = username
    if secret:
      creds.usernamePasswordCredentials.passwordSecretVersion = secret
    remote_cfg.upstreamCredentials = creds

  # MAVEN
  if repo_args.remote_mvn_repo:
    remote_cfg.mavenRepository = messages.MavenRepository()
    facade, remote_input = "Maven", repo_args.remote_mvn_repo
    enum_message = _ChoiceToRemoteEnum(facade, remote_input)
    if enum_message:  # input is PublicRepository
      remote_cfg.mavenRepository.publicRepository = enum_message
    elif _IsRemoteURI(remote_input):  # input is CustomRepository
      remote_cfg.mavenRepository.customRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigMavenRepositoryCustomRepository()
      )
      remote_cfg.mavenRepository.customRepository.uri = remote_input
    elif _IsARRemote(remote_input):  # input is ArtifactRegistryRepository
      remote_cfg.mavenRepository.artifactRegistryRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigMavenRepositoryArtifactRegistryRepository()
      )
      remote_cfg.mavenRepository.artifactRegistryRepository.repository = (
          remote_input
      )
    else:  # raise error
      _RaiseRemoteRepoUpstreamError(facade, remote_input)

  # DOCKER
  elif repo_args.remote_docker_repo:
    remote_cfg.dockerRepository = messages.DockerRepository()
    facade, remote_input = "Docker", repo_args.remote_docker_repo
    enum_message = _ChoiceToRemoteEnum(facade, remote_input)
    if enum_message:  # input is PublicRepository
      remote_cfg.dockerRepository.publicRepository = enum_message
    elif _IsRemoteURI(remote_input):  # input is CustomRepository
      remote_cfg.dockerRepository.customRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigDockerRepositoryCustomRepository()
      )
      remote_cfg.dockerRepository.customRepository.uri = remote_input
    elif _IsARRemote(remote_input):  # input is ArtifactRegistryRepository
      remote_cfg.dockerRepository.artifactRegistryRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigDockerRepositoryArtifactRegistryRepository()
      )
      remote_cfg.dockerRepository.artifactRegistryRepository.repository = (
          remote_input
      )
    else:  # raise error
      _RaiseRemoteRepoUpstreamError(facade, remote_input)

  # NPM
  elif repo_args.remote_npm_repo:
    remote_cfg.npmRepository = messages.NpmRepository()
    facade, remote_input = "Npm", repo_args.remote_npm_repo
    enum_message = _ChoiceToRemoteEnum(facade, remote_input)
    if enum_message:  # input is PublicRepository
      remote_cfg.npmRepository.publicRepository = enum_message
    elif _IsRemoteURI(remote_input):  # input is CustomRepository
      remote_cfg.npmRepository.customRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigNpmRepositoryCustomRepository()
      )
      remote_cfg.npmRepository.customRepository.uri = remote_input
    elif _IsARRemote(remote_input):  # input is ArtifactRegistryRepository
      remote_cfg.npmRepository.artifactRegistryRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigNpmRepositoryArtifactRegistryRepository()
      )
      remote_cfg.npmRepository.artifactRegistryRepository.repository = (
          remote_input
      )
    else:  # raise error
      _RaiseRemoteRepoUpstreamError(facade, remote_input)

  # PYTHON
  elif repo_args.remote_python_repo:
    remote_cfg.pythonRepository = messages.PythonRepository()
    facade, remote_input = "Python", repo_args.remote_python_repo
    enum_message = _ChoiceToRemoteEnum(facade, remote_input)
    if enum_message:  # input is PublicRepository
      remote_cfg.pythonRepository.publicRepository = enum_message
    elif _IsRemoteURI(remote_input):  # input is CustomRepository
      remote_cfg.pythonRepository.customRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigPythonRepositoryCustomRepository()
      )
      remote_cfg.pythonRepository.customRepository.uri = remote_input
    elif _IsARRemote(remote_input):  # input is ArtifactRegistryRepository
      remote_cfg.pythonRepository.artifactRegistryRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigPythonRepositoryArtifactRegistryRepository()
      )
      remote_cfg.pythonRepository.artifactRegistryRepository.repository = (
          remote_input
      )
    else:  # raise error
      _RaiseRemoteRepoUpstreamError(facade, remote_input)

  # APT
  elif repo_args.remote_apt_repo:
    remote_cfg.aptRepository = messages.AptRepository()
    facade, remote_base, remote_path = (
        "Apt",
        repo_args.remote_apt_repo,
        repo_args.remote_apt_repo_path,
    )
    enum_message = _ChoiceToRemoteEnum(facade, remote_base)
    if enum_message:  # input is PublicRepository
      remote_cfg.aptRepository.publicRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigAptRepositoryPublicRepository()
      )
      remote_cfg.aptRepository.publicRepository.repositoryBase = enum_message
      remote_cfg.aptRepository.publicRepository.repositoryPath = remote_path
    elif _IsRemoteURI(_OsPackageUri(remote_base, remote_path)):
      # input is CustomRepository
      remote_cfg.aptRepository.customRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigAptRepositoryCustomRepository()
      )
      remote_cfg.aptRepository.customRepository.uri = _OsPackageUri(
          remote_base, remote_path
      )
    elif _IsARRemote(remote_base):  # input is ArtifactRegistryRepository
      if remote_path:
        raise ar_exceptions.InvalidInputValueError(
            "--remote-apt-repo-path is not supported for Artifact Registry"
            " Repository upstream."
        )
      remote_cfg.aptRepository.artifactRegistryRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigAptRepositoryArtifactRegistryRepository()
      )
      remote_cfg.aptRepository.artifactRegistryRepository.repository = (
          remote_base
      )
    else:  # raise error
      _RaiseRemoteRepoUpstreamError(facade, remote_base)

  # YUM
  elif repo_args.remote_yum_repo:
    remote_cfg.yumRepository = messages.YumRepository()
    facade, remote_base, remote_path = (
        "Yum",
        repo_args.remote_yum_repo,
        repo_args.remote_yum_repo_path,
    )
    enum_message = _ChoiceToRemoteEnum(facade, remote_base)
    if enum_message:  # input is PublicRepository
      remote_cfg.yumRepository.publicRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigYumRepositoryPublicRepository()
      )
      remote_cfg.yumRepository.publicRepository.repositoryBase = enum_message
      remote_cfg.yumRepository.publicRepository.repositoryPath = remote_path
    elif _IsRemoteURI(_OsPackageUri(remote_base, remote_path)):
      # input is CustomRepository
      remote_cfg.yumRepository.customRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigYumRepositoryCustomRepository()
      )
      remote_cfg.yumRepository.customRepository.uri = _OsPackageUri(
          remote_base, remote_path
      )
    elif _IsARRemote(remote_base):  # input is ArtifactRegistryRepository
      if remote_path:
        raise ar_exceptions.InvalidInputValueError(
            "--remote-yum-repo-path is not supported for Artifact Registry"
            " Repository upstream."
        )
      remote_cfg.yumRepository.artifactRegistryRepository = (
          messages.GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigYumRepositoryArtifactRegistryRepository()
      )
      remote_cfg.yumRepository.artifactRegistryRepository.repository = (
          remote_base
      )
    else:  # raise error
      _RaiseRemoteRepoUpstreamError(facade, remote_base)

  else:
    return request

  request.repository.remoteRepositoryConfig = remote_cfg
  return request


def _RemoteRepoHelpText(facade: str, hide_custom_remotes: bool) -> str:
  if hide_custom_remotes:
    return """\
({facade} only) Repo upstream for {facade_lower} remote repository.
REMOTE_{command}_REPO must be one of: [{enums}].
""".format(
        facade=facade,
        facade_lower=facade.lower(),
        command=_LanguagePackageCommandName(facade),
        enums=_EnumsStrForFacade(facade),
    )
  return """\
({facade} only) Repo upstream for {facade_lower} remote repository.
REMOTE_{command}_REPO can be either:
  - one of the following enums: [{enums}].
  - an http/https custom registry uri (ex: https://my.{facade_lower}.registry)
""".format(
      facade=facade,
      facade_lower=facade.lower(),
      command=_LanguagePackageCommandName(facade),
      enums=_EnumsStrForFacade(facade),
  )


def _OsPackageRemoteRepoHelpText(facade: str, hide_custom_remotes: bool) -> str:
  if hide_custom_remotes:
    return """\
({facade} only) Repository base for {facade_lower} remote repository.
REMOTE_{facade_upper}_REPO must be one of: [{enums}].
""".format(
        facade=facade,
        facade_lower=facade.lower(),
        facade_upper=facade.upper(),
        enums=_EnumsStrForFacade(facade),
    )
  return """\
({facade} only) Repository base for {facade_lower} remote repository.
REMOTE_{facade_upper}_REPO can be either:
  - one of the following enums: [{enums}].
  - an http/https custom registry uri (ex: https://my.{facade_lower}.registry)
""".format(
      facade=facade,
      facade_lower=facade.lower(),
      facade_upper=facade.upper(),
      enums=_EnumsStrForFacade(facade),
  )


def _LanguagePackageCommandName(facade: str) -> str:
  if facade == "Maven":
    return "MVN"
  return facade.upper()


def _ChoiceToRemoteEnum(facade: str, remote_input: str):
  """Converts the remote repo input to a PublicRepository Enum message or None."""
  enums = _EnumsMessageForFacade(facade)
  name = arg_utils.ChoiceToEnumName(remote_input)
  try:
    return enums.lookup_by_name(name)
  except KeyError:
    return None


def _EnumsMessageForFacade(facade: str):
  """Returns the PublicRepository enum messages for a facade."""
  facade_to_enum = {
      "Maven": (
          ar_requests.GetMessages()
          .MavenRepository()
          .PublicRepositoryValueValuesEnum
      ),
      "Docker": (
          ar_requests.GetMessages()
          .DockerRepository()
          .PublicRepositoryValueValuesEnum
      ),
      "Npm": (
          ar_requests.GetMessages()
          .NpmRepository()
          .PublicRepositoryValueValuesEnum
      ),
      "Python": (
          ar_requests.GetMessages()
          .PythonRepository()
          .PublicRepositoryValueValuesEnum
      ),
      "Apt": (
          ar_requests.GetMessages()
          .GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigAptRepositoryPublicRepository()
          .RepositoryBaseValueValuesEnum
      ),
      "Yum": (
          ar_requests.GetMessages()
          .GoogleDevtoolsArtifactregistryV1RemoteRepositoryConfigYumRepositoryPublicRepository()
          .RepositoryBaseValueValuesEnum
      ),
  }
  return facade_to_enum[facade]


def _EnumsStrForFacade(facade: str) -> str:
  """Returns the human-readable PublicRepository enum strings for a facade."""
  return _EnumsMessageToStr(_EnumsMessageForFacade(facade))


def _EnumsMessageToStr(enums) -> str:
  """Returns the human-readable PublicRepository enum strings."""
  return ", ".join(
      arg_utils.EnumNameToChoice(name)
      for name, number in sorted(enums.to_dict().items())
      if number != 0  # Ignore UNSPECIFIED enum values.
  )


def _OsPackageUri(remote_base, remote_path):
  # Don't concatenate if remote_path not given.
  if not remote_path:
    return remote_base
  # Add '/' to end of remote_base if not already present.
  if remote_base[-1] != "/":
    remote_base = remote_base + "/"
  return remote_base + remote_path


def _IsRemoteURI(remote_input: str) -> bool:
  return remote_input.startswith("https://") or remote_input.startswith(
      "http://"
  )


def _IsARRemote(remote_input: str) -> bool:
  return remote_input.startswith("projects/")


def _RaiseRemoteRepoUpstreamError(facade: str, remote_input: str):
  raise ar_exceptions.InvalidInputValueError("""\
Invalid repo upstream for remote repository: '{remote_input}'. Valid choices are: [{enums}].
If you intended to enter a custom upstream URI, this value must start with 'https://' or 'http://'.
""".format(remote_input=remote_input, enums=_EnumsStrForFacade(facade)))



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for Cloud Platform Apis generated from apitools."""

import enum


BASE_URL = 'https://run.googleapis.com/v1/'
DOCS_URL = 'https://cloud.google.com/run/'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  API_V1_NAMESPACES = (
      'api.v1.namespaces',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}',
      },
      ['name'],
      True
  )
  API_V1_NAMESPACES_SECRETS = (
      'api.v1.namespaces.secrets',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/secrets/{secretsId}',
      },
      ['name'],
      True
  )
  NAMESPACES_CONFIGURATIONS = (
      'namespaces.configurations',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/configurations/{configurationsId}',
      },
      ['name'],
      True
  )
  NAMESPACES_DOMAINMAPPINGS = (
      'namespaces.domainmappings',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/domainmappings/{domainmappingsId}',
      },
      ['name'],
      True
  )
  NAMESPACES_EXECUTIONS = (
      'namespaces.executions',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/executions/{executionsId}',
      },
      ['name'],
      True
  )
  NAMESPACES_JOBS = (
      'namespaces.jobs',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/jobs/{jobsId}',
      },
      ['name'],
      True
  )
  NAMESPACES_REVISIONS = (
      'namespaces.revisions',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/revisions/{revisionsId}',
      },
      ['name'],
      True
  )
  NAMESPACES_ROUTES = (
      'namespaces.routes',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/routes/{routesId}',
      },
      ['name'],
      True
  )
  NAMESPACES_SERVICES = (
      'namespaces.services',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/services/{servicesId}',
      },
      ['name'],
      True
  )
  NAMESPACES_TASKS = (
      'namespaces.tasks',
      '{+name}',
      {
          '':
              'namespaces/{namespacesId}/tasks/{tasksId}',
      },
      ['name'],
      True
  )
  PROJECTS = (
      'projects',
      'projects/{projectsId}',
      {},
      ['projectsId'],
      True
  )
  PROJECTS_LOCATIONS = (
      'projects.locations',
      'projects/{projectsId}/locations/{locationsId}',
      {},
      ['projectsId', 'locationsId'],
      True
  )
  PROJECTS_LOCATIONS_CONFIGURATIONS = (
      'projects.locations.configurations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/configurations/'
              '{configurationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DOMAINMAPPINGS = (
      'projects.locations.domainmappings',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/domainmappings/'
              '{domainmappingsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_JOBS = (
      'projects.locations.jobs',
      'projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}',
      {},
      ['projectsId', 'locationsId', 'jobsId'],
      True
  )
  PROJECTS_LOCATIONS_NAMESPACES = (
      'projects.locations.namespaces',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/namespaces/'
              '{namespacesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_REVISIONS = (
      'projects.locations.revisions',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/revisions/'
              '{revisionsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_ROUTES = (
      'projects.locations.routes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/routes/'
              '{routesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_SECRETS = (
      'projects.locations.secrets',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/secrets/'
              '{secretsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_SERVICES = (
      'projects.locations.services',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/services/'
              '{servicesId}',
      },
      ['name'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Task for copying a managed folder."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import os
import threading

from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.command_lib.storage import progress_callbacks
from googlecloudsdk.command_lib.storage.tasks import task_status
from googlecloudsdk.command_lib.storage.tasks.cp import copy_util


class CopyManagedFolderTask(copy_util.CopyTaskWithExitHandler):
  """Represents a command operation copying an object around the cloud."""

  def __init__(
      self,
      source_resource,
      destination_resource,
      print_created_message=False,
      user_request_args=None,
      verbose=False,
  ):
    """Initializes CopyManagedFolderTask. Parent class documents arguments."""
    super(CopyManagedFolderTask, self).__init__(
        source_resource,
        destination_resource,
        print_created_message,
        user_request_args,
        verbose,
    )
    self.parallel_processing_key = (
        self._destination_resource.storage_url.url_string
    )

  def execute(self, task_status_queue=None):
    source_url = self._source_resource.storage_url
    destination_url = self._destination_resource.storage_url
    api_client = api_factory.get_api(source_url.scheme)

    if task_status_queue is not None:
      progress_callback = progress_callbacks.FilesAndBytesProgressCallback(
          status_queue=task_status_queue,
          offset=0,
          length=0,
          source_url=self._source_resource.storage_url,
          destination_url=self._destination_resource.storage_url,
          operation_name=task_status.OperationName.INTRA_CLOUD_COPYING,
          process_id=os.getpid(),
          thread_id=threading.get_ident(),
      )
    else:
      progress_callback = None

    source_policy = api_client.get_managed_folder_iam_policy(
        source_url.bucket_name, source_url.object_name
    )
    api_client.create_managed_folder(
        destination_url.bucket_name,
        destination_url.object_name,
    )

    self._print_created_message_if_requested(self._destination_resource)

    api_client.set_managed_folder_iam_policy(
        destination_url.bucket_name, destination_url.object_name, source_policy
    )

    if progress_callback:
      progress_callback(0)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Delete command for the Org Policy CLI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.orgpolicy import service as org_policy_service
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.org_policies import arguments
from googlecloudsdk.command_lib.org_policies import utils
from googlecloudsdk.core import log

DETAILED_HELP = {
    'DESCRIPTION':
        """\
        Deletes an organization policy.
        """,
    'EXAMPLES':
        """\
        To delete the policy associated with the constraint 'gcp.resourceLocations'
        and the Project 'foo-project', run:

        $ {command} gcp.resourceLocations --project=foo-project
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Delete(base.Command):
  """Delete an organization policy."""

  @staticmethod
  def Args(parser):
    arguments.AddConstraintArgToParser(parser)
    arguments.AddResourceFlagsToParser(parser)

  def Run(self, args):
    """Deletes an organization policy.

    The policy is deleted using DeletePolicy.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the Args method.

    Returns:
       If the policy is deleted, then messages.GoogleProtobufEmpty.
    """
    org_policy_api = org_policy_service.OrgPolicyApi(self.ReleaseTrack())
    policy_name = utils.GetPolicyNameFromArgs(args)

    delete_response = org_policy_api.DeletePolicy(policy_name)
    log.DeletedResource(policy_name, 'policy')
    return delete_response


Delete.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Policy Controller-Specific exceptions."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import exceptions


class InvalidMonitoringBackendError(exceptions.Error):
  """For when the API message for the monitoring backend isn't available.

  This may be due to a mismatch between what the API proto supports and what
  the gcloud SDK expects to support (constants.MONITORING_BACKEND).
  """


class InvalidPocoMembershipError(exceptions.Error):
  """For when the Policy Controller feature is not enabled for a membership."""


class MultiInvalidPocoMembershipsError(exceptions.MultiError):
  """For when multiple memberships do not have Policy Controller enabled."""


class MutexError(exceptions.Error):
  """For when two mutually exclusive flags are specified."""


class NoSuchContentError(exceptions.Error):
  """For when trying to configure unsupported or missing content.

  For instance, if the user attempts to install a bundle that doesn't exist or
  isn't supported, this error should be thrown.
  """


class SafetyError(exceptions.Error):
  """For when a safety check is required, but redundent.

  If this is thrown it means some other check failed. For example, a required
  argparse argument should never be None - argparse should throw an error if it
  is not passed - but safety requires we rule out the None value in later code.
  """



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Util for Fault Injection Cloud SDK."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.core import exceptions
import six


VERSION_MAP = {
    base.ReleaseTrack.ALPHA: 'v1alpha1'
}

_API_NAME = 'faultinjectiontesting'


# The messages module can also be accessed from client.MESSAGES_MODULE
def GetMessagesModule(release_track=base.ReleaseTrack.ALPHA):
  api_version = VERSION_MAP.get(release_track)
  return apis.GetMessagesModule(_API_NAME, api_version)


def GetClientInstance(release_track=base.ReleaseTrack.ALPHA):
  api_version = VERSION_MAP.get(release_track)
  return apis.GetClientInstance(_API_NAME, api_version)


class InvalidFaultConfigurationError(exceptions.Error):
  """Error if a fault configuration is improperly specified."""


class InvalidExperimentConfigurationError(exceptions.Error):
  """Error if a Experiment configuration is improperly specified."""


def ParseCreateFaultFromYaml(fault, fault_config, parent):
  """Converts the given fault dict to the corresponding import request.

  Args:
    fault: faultId, fault name
    fault_config: dict, fault configuation of the create fault request.
    parent: parent for fault resource

  Returns:
    FaultinjectiontestingProjectsLocationsFaultsCreateRequest
  Raises:
    InvalidFaultConfigurationError: If the fault config is invalid.
  """
  messages = GetMessagesModule(release_track=base.ReleaseTrack.ALPHA)
  request = messages.FaultinjectiontestingProjectsLocationsFaultsCreateRequest
  try:
    import_request_message = encoding.DictToMessage(
        {'fault': fault_config, 'faultId': fault, 'parent': parent}, request
    )
  except AttributeError:
    raise InvalidFaultConfigurationError(
        'An error occurred while parsing the '
        'serialized fault. Please check your '
        'input file.'
    )
  unrecognized_field_paths = _GetUnrecognizedFieldPaths(import_request_message)
  if unrecognized_field_paths:
    error_msg_lines = [
        'Invalid fault config, the following fields are ' + 'unrecognized:'
    ]
    error_msg_lines += unrecognized_field_paths
    raise InvalidFaultConfigurationError('\n'.join(error_msg_lines))

  return import_request_message


def ParseUpdateFaultFromYaml(fault, fault_config):
  """Converts the given fault dict to the corresponding import request.

  Args:
    fault: faultId, fault name
    fault_config: dict, fault configuation of the create fault request.

  Returns:
    FaultinjectiontestingProjectsLocationsFaultsPatchRequest
  Raises:
    InvalidFaultConfigurationError: If the fault config is invalid.
  """
  messages = GetMessagesModule(release_track=base.ReleaseTrack.ALPHA)
  request = messages.FaultinjectiontestingProjectsLocationsFaultsPatchRequest
  try:
    import_request_message = encoding.DictToMessage(
        {'fault': fault_config, 'name': fault}, request
    )
  except AttributeError:
    raise InvalidFaultConfigurationError(
        'An error occurred while parsing the '
        'serialized fault. Please check your '
        'input file.'
    )
  unrecognized_field_paths = _GetUnrecognizedFieldPaths(import_request_message)
  if unrecognized_field_paths:
    error_msg_lines = [
        'Invalid fault config, the following fields are ' + 'unrecognized:'
    ]
    error_msg_lines += unrecognized_field_paths
    raise InvalidFaultConfigurationError('\n'.join(error_msg_lines))

  return import_request_message


def ParseCreateExperimentFromYaml(experiment, experiment_config, parent):
  """Converts the given fault dict to the corresponding import request.

  Args:
    experiment: ExperimentId, Experiment name
    experiment_config: dict, experiment config of the create experiment request.
    parent: parent for experiment resource

  Returns:
    FaultinjectiontestingProjectsLocationsExperimentsCreateRequest
  Raises:
    InvalidExperimentConfigurationError: If the experiment config is invalid.
  """
  messages = GetMessagesModule(release_track=base.ReleaseTrack.ALPHA)
  req = messages.FaultinjectiontestingProjectsLocationsExperimentsCreateRequest
  try:
    import_request_message = encoding.DictToMessage(
        {
            'experiment': experiment_config,
            'experimentId': experiment,
            'parent': parent,
        },
        req,
    )
  except AttributeError:
    raise InvalidExperimentConfigurationError(
        'An error occurred while parsing the '
        'serialized experiment. Please check your '
        'input file.'
    )
  unrecognized_field_paths = _GetUnrecognizedFieldPaths(import_request_message)
  if unrecognized_field_paths:
    error_msg_lines = [
        'Invalid experiment config, the following fields are ' + 'unrecognized:'
    ]
    error_msg_lines += unrecognized_field_paths
    raise InvalidExperimentConfigurationError('\n'.join(error_msg_lines))

  return import_request_message


def ParseUpdateExperimentFromYaml(experiment, experiment_config):
  """Converts the given fault dict to the corresponding import request.

  Args:
    experiment: experimentId, experiment name
    experiment_config: dict, fault configuation of the create fault request.

  Returns:
    FaultinjectiontestingProjectsLocationsExperimentsPatchRequest
  Raises:
    InvalidExperimentConfigurationError: If the experiment config is invalid.
  """
  messages = GetMessagesModule(release_track=base.ReleaseTrack.ALPHA)
  request = (
      messages.FaultinjectiontestingProjectsLocationsExperimentsPatchRequest
  )
  try:
    import_request_message = encoding.DictToMessage(
        {'experiment': experiment_config, 'name': experiment}, request
    )
  except AttributeError:
    raise InvalidExperimentConfigurationError(
        'An error occurred while parsing the '
        'serialized experiment. Please check your '
        'input file.'
    )
  unrecognized_field_paths = _GetUnrecognizedFieldPaths(import_request_message)
  if unrecognized_field_paths:
    error_msg_lines = [
        'Invalid experiment config, the following fields are ' + 'unrecognized:'
    ]
    error_msg_lines += unrecognized_field_paths
    raise InvalidExperimentConfigurationError('\n'.join(error_msg_lines))

  return import_request_message


def _GetUnrecognizedFieldPaths(message):
  """Returns the field paths for unrecognized fields in the message."""
  errors = encoding.UnrecognizedFieldIter(message)
  unrecognized_field_paths = []
  for edges_to_message, field_names in errors:
    message_field_path = '.'.join(six.text_type(e) for e in edges_to_message)
    for field_name in field_names:
      unrecognized_field_paths.append(
          '{}.{}'.format(message_field_path, field_name)
      )
  return sorted(unrecognized_field_paths)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for adding or updating or patching partner metadata."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


from apitools.base.py import encoding
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import partner_metadata_utils
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.compute.instances import flags


DETAILED_HELP = {
    'DESCRIPTION': """\
          {command} can be used to remove instance metadata entries.
        """,
    'EXAMPLES': """\
        To remove partner metadata namespace ``gcar.googleapis.com/engine''
        and ``gcar.googleapis.com/body'' along with their data from
        an instance named ``INSTANCE_NAME'', run:

          $ {command} INSTANCE_NAME \\
          --keys=gcar.googleapis.com/engine,gcar.googleapis.com/body

        To remove all namespaces, run:
          $ {command} INSTANCE_NAME --all

        """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class InstancesRemovePartnerMetadata(base.UpdateCommand):
  """remove partner metadata namespace."""

  @staticmethod
  def Args(parser):
    flags.INSTANCE_ARG.AddArgument(
        parser, operation_type='set partner metadata on'
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '--all',
        action='store_true',
        default=False,
        help='If provided, all partner metadata namespaces are removed.',
    )
    group.add_argument(
        '--keys',
        type=arg_parsers.ArgList(min_length=1),
        metavar='KEY',
        help='The namespaces partner metadata to remove.',
    )

  def GetGetPartnerMetadataRequest(self, client, instance_ref):
    return (
        client.apitools_client.instances,
        'GetPartnerMetadata',
        client.messages.ComputeInstancesGetPartnerMetadataRequest(
            **instance_ref.AsDict()
        ),
    )

  def Run(self, args):
    if not args.all and not args.keys:
      raise calliope_exceptions.OneOfArgumentsRequiredException(
          ['--keys', '--all'],
          'One of [--all] or [--keys] must be provided.',
      )
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    instance_ref = flags.INSTANCE_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=flags.GetInstanceZoneScopeLister(client),
    )
    get_request = self.GetGetPartnerMetadataRequest(client, instance_ref)
    partner_metadata_message = client.MakeRequests([get_request])[0]
    partner_metadata_dict = encoding.MessageToDict(partner_metadata_message)[
        'partnerMetadata'
    ]
    if args.all:
      partner_metadata_dict = {k: None for k in partner_metadata_dict.keys()}
    else:
      for key in args.keys:
        namespace, *entries = key.split('/')
        if entries:
          deleted_entry = entries.pop()
          curr_dict = partner_metadata_dict[namespace]['entries']
          for entry in entries:
            curr_dict = curr_dict[entry]
          curr_dict[deleted_entry] = None
        else:
          partner_metadata_dict[namespace] = None
    partner_metadata_message = (
        partner_metadata_utils.ConvertPartnerMetadataDictToMessage(
            partner_metadata_dict
        )
    )
    patch_request = (
        client.apitools_client.instances,
        'PatchPartnerMetadata',
        client.messages.ComputeInstancesPatchPartnerMetadataRequest(
            partnerMetadata=client.messages.PartnerMetadata(
                partnerMetadata=partner_metadata_message
            ),
            **instance_ref.AsDict()
        ),
    )
    return client.MakeRequests([patch_request])


InstancesRemovePartnerMetadata.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a new workforce pool under a parent organization."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.iam import util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as gcloud_exceptions
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.iam import identity_pool_waiter
from googlecloudsdk.command_lib.iam.workforce_pools import flags
from googlecloudsdk.command_lib.util.apis import yaml_data
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import log
from googlecloudsdk.core import resources


class Create(base.CreateCommand):
  r"""Create a new workforce pool under an organization.

  Creates a workforce pool under an organization given a valid organization ID.

  ## EXAMPLES

  The following command creates a workforce pool with ID ``my-workforce-pool''
  in the organization ``12345'':

    $ {command} my-workforce-pool --organization=12345

  The following command creates a workforce pool with ID ``my-workforce-pool''
  with explicit values for all required and optional parameters:

    $ {command} my-workforce-pool --organization=12345 --location=global
    --display-name="My Workforce Pool" --description="My workforce pool
    description." --session-duration="7200s" --disabled
  """

  @staticmethod
  def Args(parser):
    workforce_pool_data = yaml_data.ResourceYAMLData.FromPath(
        'iam.workforce_pool'
    )
    concept_parsers.ConceptParser.ForResource(
        'workforce_pool',
        concepts.ResourceSpec.FromYaml(
            workforce_pool_data.GetData(), is_positional=True
        ),
        'The workforce pool to create.',
        required=True,
    ).AddToParser(parser)
    flags.AddParentFlags(parser, 'create')
    parser.add_argument(
        '--display-name',
        help=(
            'A display name for the workforce pool. Cannot exceed 32 '
            + 'characters in length.'
        ),
    )
    parser.add_argument(
        '--description',
        help=(
            'A description for the workforce pool. Cannot exceed 256 '
            + 'characters in length.'
        ),
    )
    parser.add_argument(
        '--disabled',
        action='store_true',
        help='Whether or not the workforce pool is disabled.',
    )
    parser.add_argument(
        '--session-duration',
        help=(
            'How long the Google Cloud access tokens, console sign-in '
            + 'sessions, and gcloud sign-in sessions from this workforce '
            + 'pool are valid. Must be greater than 15 minutes (900s) and '
            + 'less than 12 hours (43200s). If not configured, minted '
            + 'credentials will have a default duration of one hour (3600s).'
        ),
    )
    parser.add_argument(
        '--allowed-services',
        action='append',
        type=arg_parsers.ArgDict(spec={'domain': str}, required_keys=['domain']),
        hidden=True,
        help=(
            'Services allowed for web sign-in with the workforce pool. If not'
            ' set, by default all the services are allowed.'
        ),
    )
    parser.add_argument(
        '--disable-programmatic-signin',
        action='store_true',
        hidden=True,
        help='Disable programatic sign-in for workforce pool users.',
    )
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    client, messages = util.GetClientAndMessages()
    if not args.organization:
      raise gcloud_exceptions.RequiredArgumentException(
          '--organization',
          'Should specify the organization for workforce pools.',
      )
    parent_name = iam_util.GetParentName(
        args.organization, None, 'workforce pool'
    )
    workforce_pool_ref = args.CONCEPTS.workforce_pool.Parse()
    new_workforce_pool = messages.WorkforcePool(
        parent=parent_name,
        displayName=args.display_name,
        description=args.description,
        disabled=args.disabled,
        sessionDuration=args.session_duration,
        accessRestrictions=self.CreateAccessRestrictions(args, messages),
    )
    lro_ref = client.locations_workforcePools.Create(
        messages.IamLocationsWorkforcePoolsCreateRequest(
            location=flags.ParseLocation(args),
            workforcePoolId=workforce_pool_ref.workforcePoolsId,
            workforcePool=new_workforce_pool,
        )
    )

    log.status.Print(
        'Create request issued for: [{}]'.format(
            workforce_pool_ref.workforcePoolsId
        )
    )

    if args.async_:
      log.status.Print('Check operation [{}] for status.'.format(lro_ref.name))
      return lro_ref

    lro_resource = resources.REGISTRY.ParseRelativeName(
        lro_ref.name, collection='iam.locations.workforcePools.operations'
    )
    poller = identity_pool_waiter.IdentityPoolOperationPoller(
        client.locations_workforcePools,
        client.locations_workforcePools_operations,
    )

    # Wait for a maximum of 5 minutes, as the IAM replication has a lag of up to
    # 80 seconds. GetOperation has a dependency on IAMInternal.CheckPolicy, and
    # requires the caller to have `workforcePools.get` permission on the created
    # resource to return as `done`. See b/203589135.
    result = waiter.WaitFor(
        poller,
        lro_resource,
        'Waiting for operations [{}] to complete'.format(lro_ref.name),
        max_wait_ms=300000,
    )
    log.status.Print(
        'Created workforce pool [{}].'.format(
            workforce_pool_ref.workforcePoolsId
        )
    )

    return result

  def CreateAccessRestrictions(self, args, messages):
    if args.IsSpecified('allowed_services') or args.IsSpecified(
        'disable_programmatic_signin'
    ):
      access_restrictions = messages.AccessRestrictions()
      if args.IsSpecified('allowed_services'):
        access_restrictions.allowedServices = args.allowed_services
      if args.IsSpecified('disable_programmatic_signin'):
        access_restrictions.disableProgrammaticSignin = (
            args.disable_programmatic_signin
        )
      return access_restrictions
    return None



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Encrypt volumes under a Cloud NetApp Volumes KMS Config."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.netapp.kms_configs import client as kmsconfigs_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.netapp.kms_configs import flags as kmsconfigs_flags


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Encrypt(base.Command):
  """Encrypt all existing volumes and storage pools in the same region with the desired Cloud NetApp Volumes KMS Config."""

  detailed_help = {
      'DESCRIPTION': """\
          Encrypt the existing volumes with the desired KMS (Key Management System) Config using Customer Managed Encryption Keys (CMEK).
          """,
      'EXAMPLES': """\
          The following command encrypts the existing volumes with the desired KMS Config instance named KMS_CONFIG using specified project and location.

              $ {command} KMS_CONFIG --location=us-central1
          """,
  }

  _RELEASE_TRACK = base.ReleaseTrack.GA

  @staticmethod
  def Args(parser):
    kmsconfigs_flags.AddKMSConfigEncryptArgs(parser)

  def Run(self, args):
    """Encrypt all existing volumes and storage pools under a Cloud NetApp Volumes KMS Config in the current project."""
    kmsconfig_ref = args.CONCEPTS.kms_config.Parse()
    client = kmsconfigs_client.KmsConfigsClient(self._RELEASE_TRACK)
    return client.EncryptKmsConfig(kmsconfig_ref, args.async_)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class EncryptBeta(Encrypt):
  """Encrypt all existing volumes and storage pools in the same region with the desired Cloud NetApp Volumes KMS Config."""

  _RELEASE_TRACK = base.ReleaseTrack.BETA




# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This file provides the implementation of the `functions remove-iam-policy-binding` command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.functions.v2 import util as api_util
from googlecloudsdk.command_lib.functions import run_util
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io


def Run(args, release_track):
  """Removes a binding from the IAM policy for a Google Cloud Function."""
  client = api_util.GetClientInstance(release_track=release_track)
  messages = api_util.GetMessagesModule(release_track=release_track)

  function_ref = args.CONCEPTS.name.Parse()
  function_relative_name = function_ref.RelativeName()

  policy = client.projects_locations_functions.GetIamPolicy(
      messages.CloudfunctionsProjectsLocationsFunctionsGetIamPolicyRequest(
          resource=function_relative_name
      )
  )

  iam_util.RemoveBindingFromIamPolicy(policy, args.member, args.role)

  policy = client.projects_locations_functions.SetIamPolicy(
      messages.CloudfunctionsProjectsLocationsFunctionsSetIamPolicyRequest(
          resource=function_relative_name,
          setIamPolicyRequest=messages.SetIamPolicyRequest(policy=policy),
      )
  )

  if args.role in [
      'roles/cloudfunctions.admin',
      'roles/cloudfunctions.developer',
      'roles/cloudfunctions.invoker',
  ]:
    log.warning(
        'The binding between member {member} and role {role} has been'
        ' successfully removed. However, to make sure the member {member}'
        " doesn't have the permission to invoke the 2nd gen function, you need"
        ' to remove the invoker binding in the underlying Cloud Run service.'
        ' This can be done by running the following command:\n '
        ' gcloud functions remove-invoker-policy-binding {function_ref}'
        ' --member={member} \n'.format(
            member=args.member, role=args.role, function_ref=function_ref.Name()
        )
    )
    if console_io.CanPrompt() and console_io.PromptContinue(
        prompt_string=(
            'Would you like to run this command and additionally deny [{}]'
            ' permission to invoke function [{}]'
        ).format(args.member, function_ref.Name()),
    ):
      function = client.projects_locations_functions.Get(
          messages.CloudfunctionsProjectsLocationsFunctionsGetRequest(
              name=function_ref.RelativeName()
          )
      )
      run_util.AddOrRemoveInvokerBinding(
          function, args.member, add_binding=False
      )
      log.status.Print(
          'The role [roles/run.invoker] was successfully removed for member '
          '{member} in the underlying Cloud Run service. You can view '
          'its IAM policy by running:\n'
          'gcloud run services get-iam-policy {service}\n'.format(
              service=function.serviceConfig.service, member=args.member
          )
      )

  return policy



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""GKE Hub Policy Controller constants that are used across commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


MONITORING_BACKENDS = {
    'prometheus': 'PROMETHEUS',
    'cloudmonitoring': 'CLOUD_MONITORING',
}

INSTALL_SPEC_LABEL_MAP = {
    'INSTALL_SPEC_ENABLED': 'ENABLED',
    'INSTALL_SPEC_NOT_INSTALLED': 'NOT_INSTALLED',
    'INSTALL_SPEC_SUSPENDED': 'SUSPENDED',
    'INSTALL_SPEC_UNSPECIFIED': 'UNSPECIFIED',
}

ENFORCEMENT_ACTION_LABEL_MAP = {
    'ENFORCEMENT_ACTION_UNSPECIFIED': 'UNSPECIFIED',
    'ENFORCEMENT_ACTION_DENY': 'DENY',
    'ENFORCEMENT_ACTION_DRYRUN': 'DRYRUN',
    'ENFORCEMENT_ACTION_WARN': 'WARN',
}


def get_enforcement_action_label(enforcement_action):
  return ENFORCEMENT_ACTION_LABEL_MAP.get(enforcement_action,
                                          'ENFORCEMENT_ACTION_UNSPECIFIED')



# -*- coding: utf-8 -*- #
# Copyright 2023 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud dataproc-gdc instances create` command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.edge_cloud.container import resource_args as gdce_resource_args
from googlecloudsdk.command_lib.util.apis import yaml_data
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import log
from googlecloudsdk.core import resources

DATAPROCGDC_API_NAME = 'dataprocgdc'
DATAPROCGDC_API_VERSION = 'v1alpha1'


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Create(base.CreateCommand):
  """Create a Dataproc GDC service instance.

  A service instance is a deployment of the Dataproc operator running on a
  Kubernetes cluster. Each cluster may have at most one Dataproc service
  instance deployed. A service instance manages Application Environments
  and Spark Applications that run locally on the cluster.
  """

  detailed_help = {'EXAMPLES': """\
          To create a Dataproc GDC service instance with name `my-instance`
          in location `us-central1` running on a GDCE cluster named
          `my-cluster`, run:

          $ {command} my-instance --location=us-central1 --gdce-cluster=my-cluster

          Note that the GDCE cluster and the Dataproc GDC service instance must
          be in the same project and Cloud location (in this case us-central1).
          """}

  @staticmethod
  def Args(parser):
    concept_parsers.ConceptParser(
        [
            GetInstanceResourcePresentationSpec(),
            GetGdceClusterResourcePresentationSpec(),
        ],
        command_level_fallthroughs={
            # Set the GDCE cluster location to the instance location
            '--gdce-cluster.location': ['instance.location']
        },
    ).AddToParser(parser)
    parser.add_argument(
        '--request-id',
        help="""An optional request ID to identify requests. If the service receives two identical
        instance create requests with the same request_id, the second request is
        ignored and the operation that corresponds to the first request is returned for both.

        The request ID must be a valid UUID with the exception that zero UUID is
        not supported (00000000-0000-0000-0000-000000000000).""",
    )
    parser.add_argument(
        '--display-name',
        help=(
            'Human-readable name for this service instance to be used in user'
            ' interfaces.'
        ),
    )
    parser.add_argument(
        '--annotations',
        metavar='KEY=VALUE',
        type=arg_parsers.ArgDict(),
        action=arg_parsers.UpdateAction,
        help=(
            'List of annotation KEY=VALUE pairs to add to the service instance.'
        ),
    )
    labels_util.AddCreateLabelsFlags(parser)
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    dataprocgdc_client = apis.GetClientInstance(
        DATAPROCGDC_API_NAME, DATAPROCGDC_API_VERSION
    )
    messages = apis.GetMessagesModule('dataprocgdc', 'v1alpha1')
    instance_ref = args.CONCEPTS.instance.Parse()
    cluster_ref = args.CONCEPTS.gdce_cluster.Parse()

    if args.annotations:
      annotations = encoding.DictToAdditionalPropertyMessage(
          args.annotations,
          messages.ServiceInstance.AnnotationsValue,
          sort_items=True,
      )
    else:
      annotations = None

    create_req = (
        messages.DataprocgdcProjectsLocationsServiceInstancesCreateRequest(
            serviceInstanceId=instance_ref.Name(),
            parent=instance_ref.Parent().RelativeName(),
            requestId=args.request_id,
            serviceInstance=messages.ServiceInstance(
                displayName=args.display_name,
                labels=labels_util.ParseCreateArgs(
                    args, messages.ServiceInstance.LabelsValue
                ),
                annotations=annotations,
                gdceCluster=messages.GdceCluster(
                    gdceCluster=cluster_ref.RelativeName()
                ),
            ),
        )
    )

    create_op = dataprocgdc_client.projects_locations_serviceInstances.Create(
        create_req
    )

    async_ = getattr(args, 'async_', False)
    if not async_:
      # Poll for operation
      operation_ref = resources.REGISTRY.Parse(
          create_op.name, collection='dataprocgdc.projects.locations.operations'
      )
      poller = waiter.CloudOperationPoller(
          dataprocgdc_client.projects_locations_serviceInstances,
          dataprocgdc_client.projects_locations_operations,
      )
      waiter.WaitFor(
          poller,
          operation_ref,
          'Waiting for service instance create operation [{0}]'.format(
              operation_ref.RelativeName()
          ),
      )
      log.CreatedResource(
          instance_ref.Name(),
          details=(
              '- service instance created in [{0}] for cluster [{1}]'.format(
                  instance_ref.Parent().RelativeName(),
                  cluster_ref.RelativeName(),
              )
          ),
      )
      return

    log.status.Print(
        'Create request issued for: [{0}]\nCheck operation [{1}] for status.'
        .format(instance_ref.Name(), create_op.name)
    )


def GetInstanceResourcePresentationSpec():
  instance_data = yaml_data.ResourceYAMLData.FromPath(
      'dataproc_gdc.service_instance'
  )
  resource_spec = concepts.ResourceSpec.FromYaml(instance_data.GetData())
  return presentation_specs.ResourcePresentationSpec(
      name='instance',
      concept_spec=resource_spec,
      group_help='Name of the service instance to create.',
      required=True,
      prefixes=False,
  )


def GetGdceClusterResourcePresentationSpec():
  return presentation_specs.ResourcePresentationSpec(
      name='--gdce-cluster',
      concept_spec=gdce_resource_args.GetClusterResourceSpec(),
      group_help='The GDCE cluster on which to create the service instance.',
      required=True,
      prefixes=True,
      # This hides the location flag for the GDCE cluster resource, since we
      # always enforce that the cluster is the same region as the instance
      flag_name_overrides={'location': ''},
  )



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command group for the Resource Manager CLI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Tags(base.Group):
  """Create and manipulate tag keys, values and bindings.

    The Tags Service gives you centralized and programmatic
    control over your organization's tags. As the tag
    administrator, you will be able to create and configure restrictions across
    the tags in your organization or projects. As the tag user, you will be able
    to attach tags values to different resources.
  """



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Storage client-side error classes."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import exceptions as core_exceptions


class Error(core_exceptions.Error):
  """Base exception for command_lib.storage modules."""


class FatalError(Error):
  """Error raised when future execution should stop."""


class HashMismatchError(Error):
  """Error raised when hashes don't match after operation."""


class InvalidPythonVersionError(Error):
  """Error raised for an invalid Python version."""


class InvalidUrlError(Error):
  """Error raised when the url string is not in the expected format."""


class SystemPermissionError(Error):
  """Error raised when encountering a systems-permissions-related issue."""


class ValueCannotBeDeterminedError(Error):
  """Error raised when attempting to access unknown information."""



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Sets configuration properties of the Policy Controller component deployments."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container.fleet.policycontroller import protos
from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.command_lib.container.fleet.features import base
from googlecloudsdk.command_lib.container.fleet.policycontroller import command
from googlecloudsdk.command_lib.container.fleet.policycontroller import deployment_configs as deployment
from googlecloudsdk.command_lib.container.fleet.policycontroller import flags
from googlecloudsdk.core import exceptions


@calliope_base.Hidden
@calliope_base.ReleaseTracks(
    calliope_base.ReleaseTrack.ALPHA, calliope_base.ReleaseTrack.BETA
)
class Set(base.UpdateCommand, command.PocoCommand):
  """Sets configuration of the Policy Controller components.

  Customizes on-cluster components of Policy Controller. Supported
  properties may be set with this command, or removed with 'remove'. These
  components are managed as individual kubernetes deployments (e.g. 'admission')
  in the gatekeeper-system namespace.

  When setting cpu or memory limits and requests, Kubernetes-standard resource
  units are used.

  All properties set using this command will overwrite previous properties, with
  the exception of tolerations which can only be added, and any number may be
  added. To edit a toleration, use 'remove' to first delete it, and then 'set'
  the desired toleration.

  ## EXAMPLES

  To set the replica count for a component:

    $ {command} admission replica-count 3

  To set the replica count for a component across all fleet memberships:

    $ {command} admission replica-count 3 --all-memberships

  To set a toleration with key 'my-key' on a component (which is an 'Exists'
  operator):

    $ {command} admission toleration my-key

  To set a toleration with key 'my-key' and 'my-value' on a component (which is
  an 'Equal' operator):

    $ {command} admission toleration my-key=my-value

  To set a toleration with key 'my-key' and 'my-value' on a component, along
  with the effect 'NoSchedule' (which is an 'Equal' operator):

    $ {command} admission toleration my-key=my-value --effect=NoSchedule

  To set a memory limit:

    $ {command} audit memory-limit 4Gi

  To set a memory request:

    $ {command} mutation memory-request 2Gi

  To set a cpu limit:

    $ {command} admission cpu-limit 500m

  To set a cpu request:

    $ {command} audit cpu-request 250m
  """

  feature_name = 'policycontroller'

  @classmethod
  def Args(cls, parser):
    cmd_flags = flags.PocoFlags(parser, 'set deployment configuration')
    cmd_flags.add_memberships()

    parser.add_argument(
        'deployment',
        choices=deployment.G8R_COMPONENTS,
        help=(
            'The PolicyController deployment component (e.g. "admission", '
            ' "audit" or "mutation") upon which to set configuration.'
        ),
    )
    parser.add_argument(
        'property',
        choices=deployment.SUPPORTED_PROPERTIES,
        help='Property to be set.',
    )
    parser.add_argument(
        'value',
        help=(
            'The value to set the property to. Valid input varies'
            ' based on the property being set.'
        ),
    )
    parser.add_argument(
        '--effect',
        choices=deployment.K8S_SCHEDULING_OPTIONS,
        type=str,
        help='Applies only to "toleration" property.',
    )

  def Run(self, args):
    # All the membership specs for this feature.
    specs = self.path_specs(args)

    for _, spec in specs.items():
      cfgs = protos.additional_properties_to_dict(
          spec.policycontroller.policyControllerHubConfig.deploymentConfigs
      )
      deployment_cfg = cfgs.get(
          args.deployment,
          self.messages.PolicyControllerPolicyControllerDeploymentConfig(),
      )

      cfgs[args.deployment] = self.set_deployment_config(
          deployment_cfg,
          args.property,
          args.value,
          args.effect,
      )

      # Convert back to a list of additionalProperties.
      dcv = protos.set_additional_properties(
          self.messages.PolicyControllerHubConfig.DeploymentConfigsValue(), cfgs
      )

      spec.policycontroller.policyControllerHubConfig.deploymentConfigs = dcv

    return self.merge_specs(specs)

  def set_deployment_config(self, deployment_cfg, prop, value, effect):
    if prop == 'toleration':
      return deployment.add_toleration(
          self.messages, deployment_cfg, value, effect
      )
    if effect is not None:
      raise exceptions.Error(
          '"effect" flag only accepted when setting a toleration.'
      )
    if prop == 'cpu-limit':
      return deployment.update_cpu_limit(self.messages, deployment_cfg, value)
    if prop == 'cpu-request':
      return deployment.update_cpu_request(self.messages, deployment_cfg, value)
    if prop == 'memory-limit':
      return deployment.update_mem_limit(self.messages, deployment_cfg, value)
    if prop == 'memory-request':
      return deployment.update_mem_request(self.messages, deployment_cfg, value)
    if prop == 'replica-count':
      return deployment.update_replica_count(deployment_cfg, value)

  def merge_specs(self, specs):
    orig = self.hubclient.ToPyDict(self.GetFeature().membershipSpecs)
    merged = {path: specs.get(path, spec) for path, spec in orig.items()}
    self.Update(
        ['membership_specs'],
        self.messages.Feature(
            membershipSpecs=self.hubclient.ToMembershipSpecs(merged)
        ),
    )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Marketplace Solutions client."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


import io
import json
import re


from apitools.base.py import exceptions as apitools_exceptions
from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import exceptions as apilib_exceptions
from googlecloudsdk.calliope.parser_errors import DetailedArgumentError
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import resources
from googlecloudsdk.core.resource import resource_printer
import six

_REGIONAL_IAM_REGEX = re.compile(
    "PERMISSION_DENIED: Permission (.+) denied on 'projects/(.+?)/.*")
_DEFAULT_API_VERSION = 'v1alpha1'
_GLOBAL_REGION = 'global'

_PFORG = 'pforg'
_ALLOWED_PRODUCTS = [_PFORG]


def _ValidateProduct(product):
  """Validates product property. Returns custom error message if invalid."""
  if product in _ALLOWED_PRODUCTS:
    pass
  else:
    raise DetailedArgumentError('Allowed products are %s' %
                                json.dumps(_ALLOWED_PRODUCTS))


def _ParseError(error):
  """Returns a best-effort error message created from an API client error."""
  if isinstance(error, apitools_exceptions.HttpError):
    parsed_error = apilib_exceptions.HttpException(error,
                                                   error_format='{message}')
    error_message = parsed_error.message
  else:
    error_message = six.text_type(error)
  return error_message


def _CollapseRegionalIAMErrors(errors):
  """If all errors are PERMISSION_DENIEDs, use a single global error instead."""
  if errors:
    matches = [_REGIONAL_IAM_REGEX.match(e) for e in errors]
    if (all(match is not None for match in matches)
        and len(set(match.group(1) for match in matches)) == 1):
      errors = ['PERMISSION_DENIED: Permission %s denied on projects/%s' %
                (matches[0].group(1), matches[0].group(2))]
  return errors


# TODO(b/271949365) Add support for aggregated list
class MpsClient(object):
  """Cloud Marketplace Solutions client."""

  def __init__(self, api_version=_DEFAULT_API_VERSION):
    self._client = apis.GetClientInstance('marketplacesolutions', api_version)
    self._messages = apis.GetMessagesModule('marketplacesolutions', api_version)

    # pylint: disable-next=line-too-long
    self.power_instances_service = self._client.projects_locations_powerInstances
    self.power_volumes_service = self._client.projects_locations_powerVolumes
    self.power_images_service = self._client.projects_locations_powerImages
    self.power_networks_service = self._client.projects_locations_powerNetworks
    self.power_sshkeys_service = self._client.projects_locations_powerSshKeys
    self.operation_service = self._client.projects_locations_operations
    self.locations_service = self._client.projects_locations

    self.power_instance_vitual_cpu_type_to_message = {
        'UNSPECIFIED': self.messages.PowerInstance.
                       VirtualCpuTypeValueValuesEnum.VIRTUAL_CPU_TYPE_UNSPECIFIED,
        'DEDICATED': self.messages.PowerInstance.
                     VirtualCpuTypeValueValuesEnum.DEDICATED,
        'UNCAPPED_SHARED': self.messages.PowerInstance.
                           VirtualCpuTypeValueValuesEnum.UNCAPPED_SHARED,
        'CAPPED_SHARED': self.messages.PowerInstance.
                         VirtualCpuTypeValueValuesEnum.CAPPED_SHARED,
    }

  @property
  def client(self):
    return self._client

  @property
  def messages(self):
    return self._messages

  def AggregateYieldFromList(self,
                             service,
                             project_resource,
                             request_class,
                             resource,
                             global_params=None,
                             limit=None,
                             method='List',
                             predicate=None,
                             skip_global_region=True,
                             allow_partial_server_failure=True):
    """Make a series of List requests, across locations in a project.

    Args:
      service: apitools_base.BaseApiService, A service with a .List() method.
      project_resource: str, The resource name of the project.
      request_class: class, The class type of the List RPC request.
      resource: string, The name (in plural) of the resource type.
      global_params: protorpc.messages.Message, The global query parameters to
        provide when calling the given method.
      limit: int, The maximum number of records to yield. None if all available
        records should be yielded.
      method: str, The name of the method used to fetch resources.
      predicate: lambda, A function that returns true for items to be yielded.
      skip_global_region: bool, True if global region must be filtered out while
      iterating over regions
      allow_partial_server_failure: bool, if True don't fail and only print a
        warning if some requests fail as long as at least one succeeds. If
        False, fail the complete command if at least one request fails.

    Yields:
      protorpc.message.Message, The resources listed by the service.

    """
    response_count = 0
    errors = []
    for location in self.ListLocations(project_resource):
      # TODO(b/198857865): Global region will be used when it is ready.
      location_name = location.name.split('/')[-1]
      if skip_global_region and location_name == _GLOBAL_REGION:
        continue
      request = request_class(parent=location.name)
      try:
        response = getattr(service, method)(
            request, global_params=global_params)
        response_count += 1
      except Exception as e:  # pylint: disable=broad-except
        errors.append(_ParseError(e))
        continue
      items = getattr(response, resource)
      if predicate:
        items = list(filter(predicate, items))
      for item in items:
        yield item
        if limit is None:
          continue
        limit -= 1
        if not limit:
          break

    if errors:
      # If the command allows partial server errors, instead of raising an
      # exception to show something went wrong, we show a warning message that
      # contains the error messages instead.
      buf = io.StringIO()
      fmt = ('list[title="Some requests did not succeed.",'
             'always-display-title]')
      if allow_partial_server_failure and response_count > 0:
        resource_printer.Print(sorted(set(errors)), fmt, out=buf)
        log.warning(buf.getvalue())
      else:
        # If all requests failed, clean them up if they're duplicated IAM errors
        collapsed_errors = _CollapseRegionalIAMErrors(errors)
        resource_printer.Print(sorted(set(collapsed_errors)), fmt, out=buf)
        raise exceptions.Error(buf.getvalue())

  def ListLocations(self,
                    project_resource,
                    limit=None,
                    page_size=None):
    """Make a List Locations request."""
    request = self.messages.MarketplacesolutionsProjectsLocationsListRequest(
        name='projects/' + project_resource)
    return list_pager.YieldFromList(
        self.locations_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='locations')

  def AggregateListInstances(self, project_resource, product, limit=None):
    """Make a series of List Instance requests."""
    _ValidateProduct(product)
    if product == _PFORG:
      power_resource = 'powerInstances'
      return self.AggregateYieldFromList(
          self.power_instances_service,
          project_resource,
          self.messages.MarketplacesolutionsProjectsLocationsPowerInstancesListRequest,
          power_resource,
          limit=limit,
      )

  def GetInstance(self, product, resource):
    """Make a Get Instance request. Return details of specified instance."""
    _ValidateProduct(product)
    resource = resource.RelativeName()
    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerInstancesGetRequest(
          name=resource
      )
      return self.power_instances_service.Get(power_request)

  def ListInstances(self, product, location_resource):
    """Make a List Instances request. Return list of instances."""
    _ValidateProduct(product)
    location = location_resource.RelativeName()

    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerInstancesListRequest(
          parent=location
      )
      return self.power_instances_service.List(power_request).powerInstances

  def ParseNetworkAttachments(self, location, project, network_attachment):
    """Parse network attachments in flag to create network list."""
    # pylint: disable=line-too-long
    networks = []
    for net in network_attachment:
      power_network = resources.REGISTRY.Parse(
          net,
          params={
              'projectsId': project.Name(),
              'locationsId': location.Name(),
          },
          collection='marketplacesolutions.projects.locations.powerNetworks').RelativeName()
      networks.append(self.messages.NetworkAttachment(powerNetwork=power_network))
    # pylint: enable=line-too-long
    return networks

  def CreateInstance(self, product,
                     instance_resource,
                     boot_image_name,
                     system_type,
                     memory_gib,
                     network_attachment_names,
                     virtual_cpu_cores,
                     virtual_cpu_type):
    """Create an Instance resource."""
      # pylint: disable=line-too-long
      # pylint: disable=bad-indentation
    _ValidateProduct(product)
    if product == _PFORG:
      location = instance_resource.Parent()
      project = location.Parent()
      boot_image = resources.REGISTRY.Parse(
          boot_image_name,
          params={
              'projectsId': project.Name(),
              'locationsId': location.Name(),
          },
          collection='marketplacesolutions.projects.locations.powerImages').RelativeName()

      instance_msg = self.messages.PowerInstance(
          name=instance_resource.RelativeName(),
          bootImage=boot_image,
          memoryGib=memory_gib,
          networkAttachments=self.ParseNetworkAttachments(location, project, network_attachment_names),
          systemType=system_type,
          virtualCpuCores=virtual_cpu_cores,
          virtualCpuType=self.power_instance_vitual_cpu_type_to_message[
              virtual_cpu_type])
      instance_id = instance_resource.RelativeName().split('/')[-1]
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerInstancesCreateRequest(
          powerInstance=instance_msg,
          powerInstanceId=instance_id,
          parent=instance_resource.Parent().RelativeName())
      return self.power_instances_service.Create(power_request)
    # pylint: enable=line-too-long
    # pylint: enable=bad-indentation

  def DeleteInstance(self, product, instance_resource):
    """Delete an existing instance share resource."""
    # pylint: disable=line-too-long
    if product == _PFORG:
      request = self.messages.MarketplacesolutionsProjectsLocationsPowerInstancesDeleteRequest(
          name=instance_resource.RelativeName())
      return self.power_instances_service.Delete(request)
    # pylint: enable=line-too-long

  def UpdateInstance(self, product, instance_resource,
                     memory_gib, virtual_cpu_cores):
    """Update an existing instance share resource."""
    updated_fields = []
    if memory_gib is not None:
      updated_fields.append('memory_gib')
    if virtual_cpu_cores is not None:
      updated_fields.append('virtual_cpu_cores')

    if product == _PFORG:
      instance_msg = self.messages.PowerInstance(
          name=instance_resource.RelativeName(),
          memoryGib=memory_gib,
          virtualCpuCores=virtual_cpu_cores)
      # pylint: disable=line-too-long
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerInstancesPatchRequest(
          name=instance_resource.RelativeName(), powerInstance=instance_msg, updateMask=','.join(updated_fields))
      return self.power_instances_service.Patch(power_request)

  def AggregateListVolumes(self, project_resource, product, limit=None):
    """Make a series of List Volume requests."""
    _ValidateProduct(product)
    if product == _PFORG:
      power_resource = 'powerVolumes'
      return self.AggregateYieldFromList(
          self.power_volumes_service,
          project_resource,
          self.messages.MarketplacesolutionsProjectsLocationsPowerVolumesListRequest,
          power_resource,
          limit=limit,
      )

  def GetVolume(self, product, resource):
    """Make a Get Volume request. Return details of specified volume."""
    _ValidateProduct(product)
    resource = resource.RelativeName()
    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerVolumesGetRequest(
          name=resource
      )
      return self.power_volumes_service.Get(power_request)

  def ListVolumes(self, product, location_resource):
    """Make a List Volumes request. Return list of volumes."""
    _ValidateProduct(product)
    location = location_resource.RelativeName()
    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerVolumesListRequest(
          parent=location
      )
      return self.power_volumes_service.List(power_request).powerVolumes

  def AggregateListImages(self, project_resource, product, limit=None):
    """Make a series of List Image requests."""
    _ValidateProduct(product)
    if product == _PFORG:
      power_resource = 'powerImages'
      return self.AggregateYieldFromList(
          self.power_images_service,
          project_resource,
          self.messages.MarketplacesolutionsProjectsLocationsPowerImagesListRequest,
          power_resource,
          limit=limit,
      )

  def GetImage(self, product, resource):
    """Make a Get Image request. Return details of specified image."""
    _ValidateProduct(product)
    resource = resource.RelativeName()
    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerImagesGetRequest(
          name=resource
      )
      return self.power_images_service.Get(power_request)

  def ListImages(self, product, location_resource):
    """Make a List Images request. Return list of images."""
    _ValidateProduct(product)
    location = location_resource.RelativeName()
    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerImagesListRequest(
          parent=location
      )
      return self.power_images_service.List(power_request).powerImages

  def AggregateListNetworks(self, project_resource, product, limit=None):
    """Make a series of List Networks requests."""
    _ValidateProduct(product)
    if product == _PFORG:
      power_resource = 'powerNetworks'
      return self.AggregateYieldFromList(
          self.power_networks_service,
          project_resource,
          self.messages.MarketplacesolutionsProjectsLocationsPowerNetworksListRequest,
          power_resource,
          limit=limit,
      )

  def GetNetwork(self, product, resource):
    """Make a Get Network request. Return details of specified network."""
    _ValidateProduct(product)
    resource = resource.RelativeName()
    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerNetworksGetRequest(
          name=resource
      )
      return self.power_networks_service.Get(power_request)

  def ListNetworks(self, product, location_resource):
    """Make a List Networks request. Return list of networks."""
    _ValidateProduct(product)
    location = location_resource.RelativeName()
    try:
      if product == _PFORG:
        power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerNetworksListRequest(
            parent=location
        )
        return self.power_networks_service.List(power_request).powerNetworks
    except exceptions.Error as e:
      return e

  def AggregateListSSHKeys(self, project_resource, product, limit=None):
    """Make a series of List SSH keys requests."""
    _ValidateProduct(product)
    if product == _PFORG:
      power_resource = 'powerSshKeys'
      return self.AggregateYieldFromList(
          self.power_sshkeys_service,
          project_resource,
          self.messages.MarketplacesolutionsProjectsLocationsPowerSshKeysListRequest,
          power_resource,
          limit=limit,
      )

  def GetSSHKey(self, product, resource):
    """Make a Get SSH Key request. Return details of specified SSH key."""
    _ValidateProduct(product)
    resource = resource.RelativeName()
    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerSshKeysGetRequest(
          name=resource
      )
      return self.power_sshkeys_service.Get(power_request)

  def ListSSHKeys(self, product, location_resource):
    """Make a List SSH Keys request. Return list of SSH keys."""
    _ValidateProduct(product)
    location = location_resource.RelativeName()
    if product == _PFORG:
      power_request = self.messages.MarketplacesolutionsProjectsLocationsPowerSshKeysListRequest(
          parent=location
      )
      return self.power_sshkeys_service.List(power_request).powerSshKeys



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update an ekmconnection."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.kms import certs
from googlecloudsdk.command_lib.kms import exceptions as kms_exceptions
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.command_lib.kms import maps
from googlecloudsdk.command_lib.kms import resource_args


class Update(base.UpdateCommand):
  r"""Update an ekmconnection.

  {command} can be used to update the ekmconnection. Updates can be made to the
  ekmconnection's service resolver's fields.

  ## EXAMPLES

  The following command updates an ekm-connection named `laplace` service
  resolver's hostname within location `us-east1`:

  $ {command} laplace --location=us-east1 \
                      --hostname=newhostname.foo

  The following command updates an ekm-connection named `laplace` service
  resolver's service_directory_service, endpoint_filter, hostname, and
  server_certificates within location `us-east1`:

    $ {command} laplace --location=us-east1 \
        --service-directory-service="foo" \
        --endpoint-filter="foo > bar" \
        --hostname="newhostname.foo" \
        --server-certificates-files=foo.pem,bar.pem

  The following command updates an ekm-connection named `laplace`
  key_management_mode within location `us-east1`:

  $ {command} laplace --location=us-east1 \
                      --key-management-mode=manual

  """

  @staticmethod
  def Args(parser):
    resource_args.AddKmsEkmConnectionResourceArgForKMS(parser, True,
                                                       'ekm_connection')
    flags.AddServiceDirectoryServiceFlag(parser)
    flags.AddEndpointFilterFlag(parser)
    flags.AddHostnameFlag(parser)
    flags.AddKeyManagementModeFlags(parser)
    flags.AddServerCertificatesFilesFlag(parser)

  def CreateUpdateMask(self, args):
    update_mask = []
    if (args.service_directory_service or args.endpoint_filter or
        args.hostname or args.server_certificates_files):
      update_mask.append('serviceResolvers')
    if args.key_management_mode:
      update_mask.append('keyManagementMode')
    if args.crypto_space_path:
      update_mask.append('cryptoSpacePath')
    return ','.join(update_mask)

  def CreateRequest(self, args, messages, ekm_connection_to_update):
    ec_ref = flags.ParseEkmConnectionName(args)
    service_resolver_to_update = ekm_connection_to_update.serviceResolvers[0]

    if args.service_directory_service:
      service_resolver_to_update.serviceDirectoryService = args.service_directory_service

    if args.endpoint_filter:
      service_resolver_to_update.endpointFilter = args.endpoint_filter

    if args.hostname:
      service_resolver_to_update.hostname = args.hostname

    if args.key_management_mode:
      ekm_connection_to_update.keyManagementMode = (
          maps.KEY_MANAGEMENT_MODE_MAPPER.GetEnumForChoice(
              args.key_management_mode))

    if args.crypto_space_path:
      ekm_connection_to_update.cryptoSpacePath = args.crypto_space_path

    certificate_list = []
    if args.server_certificates_files:
      for cert_file in args.server_certificates_files:
        try:
          certificate_list.append(
              messages.Certificate(rawDer=certs.GetDerCertificate(cert_file)))
        except Exception as e:
          raise exceptions.BadArgumentException(
              '--server-certificates-files',
              'Error while attempting to read file {} : {}'.format(
                  cert_file, e))
      service_resolver_to_update.serverCertificates = certificate_list

    req = messages.CloudkmsProjectsLocationsEkmConnectionsPatchRequest(
        name=ec_ref.RelativeName(), ekmConnection=ekm_connection_to_update)

    req.updateMask = self.CreateUpdateMask(args)

    return req

  def Run(self, args):
    if not (args.service_directory_service or args.endpoint_filter or
            args.hostname or args.server_certificates_files or
            args.key_management_mode or args.crypto_space_path):
      raise kms_exceptions.UpdateError(
          'An error occured: At least one of --service-directory-service or '
          '--endpoint-filter or --hostname or --server-certificates-files or '
          '--key-management-mode or --crypto-space-path must be specified.')

    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()
    ec_ref = flags.ParseEkmConnectionName(args)

    # Try to get the ekmConnection and raise an exception if it doesn't exist.
    ekm_connection = client.projects_locations_ekmConnections.Get(
        messages.CloudkmsProjectsLocationsEkmConnectionsGetRequest(
            name=ec_ref.RelativeName()))

    # Make update request
    update_req = self.CreateRequest(args, messages, ekm_connection)

    return client.projects_locations_ekmConnections.Patch(update_req)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility functions for Functions specific to deploying Gen2 functions."""


from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.api_lib.functions.v2 import util as api_util
from googlecloudsdk.command_lib.projects import util as projects_util


def ensure_pubsub_sa_has_token_creator_role():
  """Ensures the project's Pub/Sub service account has permission to create tokens.

  If the permission is missing, prompts the user to grant it. If the console
  cannot prompt, prints a warning instead.
  """
  pubsub_sa = 'service-{}@gcp-sa-pubsub.iam.gserviceaccount.com'.format(
      projects_util.GetProjectNumber(api_util.GetProject())
  )

  api_util.PromptToBindRoleIfMissing(
      pubsub_sa,
      'roles/iam.serviceAccountTokenCreator',
      alt_roles=['roles/pubsub.serviceAgent'],
      reason=(
          'Pub/Sub needs this role to create identity tokens. '
          'For more details, please see '
          'https://cloud.google.com/pubsub/docs/push#authentication'
      ),
  )


def ensure_data_access_logs_are_enabled(trigger_event_filters):
  # type: (list[cloudfunctions_v2_messages.EventFilter]) -> None
  """Ensures appropriate Data Access Audit Logs are enabled for the given event filters.

  If they're not, the user will be prompted to enable them or warned if the
  console cannot prompt.

  Args:
    trigger_event_filters: the CAL trigger's event filters.
  """
  service_filter = [
      f for f in trigger_event_filters if f.attribute == 'serviceName'
  ]
  if service_filter:
    api_util.PromptToEnableDataAccessAuditLogs(service_filter[0].value)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to set-iam-policy for a Dataplex lake resource."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.dataplex import datascan
from googlecloudsdk.api_lib.util import exceptions as gcloud_exception
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataplex import resource_args
from googlecloudsdk.command_lib.iam import iam_util


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class SetIamPolicy(base.Command):
  """Set the IAM policy to a Dataplex datascan as defined in a JSON or YAML file.

  See https://cloud.google.com/iam/docs/managing-policies for details of
  the policy file format and contents.
  """

  detailed_help = {
      'EXAMPLES': """\
          The following command will read an IAM policy defined in a JSON file
          `policy.son` and set it for the Dataplex datascan `test-datascan` defined in
          location `us-central1`:

            $ {command} --location=us-central1 test-datascan policy.json

          """,
  }

  @staticmethod
  def Args(parser):
    resource_args.AddDatascanResourceArg(
        parser, 'to set IAM policy binding to.'
    )
    iam_util.AddArgForPolicyFile(parser)

  @gcloud_exception.CatchHTTPErrorRaiseHTTPException(
      'Status code: {status_code}. {status_message}.'
  )
  def Run(self, args):
    datascan_ref = args.CONCEPTS.datascan.Parse()
    result = datascan.SetIamPolicyFromFile(datascan_ref, args.policy_file)
    return result



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for proxying to a given service."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.config import config_helper
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import exceptions
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import messages_util
from googlecloudsdk.command_lib.run import platforms
from googlecloudsdk.command_lib.run import pretty_print
from googlecloudsdk.command_lib.run import proxy
from googlecloudsdk.command_lib.run import resource_args
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core.credentials import store


class Proxy(base.BinaryBackedCommand):
  """Proxy a service to localhost authenticating as the active account or with the specified token."""

  detailed_help = {
      'DESCRIPTION':
          """\
          {description}
          """,
      'EXAMPLES':
          """\
          To proxy the service 'my-service' at localhost port 8080:

              $ {command} my-service --port=8080

          To proxy the existing traffic tag 'my-tag' on the service 'my-service:

              $ {command} my-service --tag=my-tag
          """,
  }

  @staticmethod
  def CommonArgs(parser):
    service_presentation = presentation_specs.ResourcePresentationSpec(
        'SERVICE',
        resource_args.GetServiceResourceSpec(),
        'Service to proxy locally.',
        required=True,
        prefixes=False)
    flags.AddPortFlag(
        parser,
        help_text='Local port number to expose the proxied service. '
        'If not specified, it will be set to 8080.')
    flags.AddTokenFlag(parser)
    flags.AddDeployTagFlag(
        parser,
        help_text='Traffic tag of the service to expose via the proxy. If not '
        'specified, the default service URL will be proxied which may '
        'serve different revisions based on traffic-splits. '
        'Custom tags can be used to proxy specific revisions. Please see '
        'https://cloud.google.com/run/docs/rollouts-rollbacks-traffic-migration#tags.'
    )
    concept_parsers.ConceptParser([service_presentation]).AddToParser(parser)

  @staticmethod
  def Args(parser):
    Proxy.CommonArgs(parser)

  def _CheckPlatform(self):
    platform = platforms.GetPlatform()
    if platform != platforms.PLATFORM_MANAGED:
      raise exceptions.PlatformError(
          'This command is only supported for fully managed Cloud Run.')

  def Run(self, args):
    self._CheckPlatform()

    conn_context = connection_context.GetConnectionContext(
        args, flags.Product.RUN, self.ReleaseTrack())
    service_ref = args.CONCEPTS.service.Parse()
    flags.ValidateResource(service_ref)
    with serverless_operations.Connect(conn_context) as client:
      serv = client.GetService(service_ref)
    if not serv:
      raise exceptions.ArgumentError(
          messages_util.GetNotFoundMessage(conn_context, service_ref)
      )

    bind = '127.0.0.1:' + (args.port if args.port else '8080')
    host = self._GetUrl(serv, args.tag, service_ref.servicesId)

    command_executor = proxy.ProxyWrapper()
    pretty_print.Info(
        messages_util.GetStartDeployMessage(
            conn_context,
            service_ref,
            'Proxying to',
        )
    )
    pretty_print.Info('http://{} proxies to {}'.format(bind, host))

    if args.token:
      response = command_executor(host=host, token=args.token, bind=bind)
    else:
      # Keep restarting the proxy with fresh token before the token expires (1h)
      # until hitting a failure.
      while True:
        response = command_executor(
            host=host, token=_GetFreshIdToken(), bind=bind, duration='55m')
        if response.failed:
          break

    return self._DefaultOperationResponseHandler(response)

  def _GetUrl(self, serv, tag, serv_id):
    if not serv.status:
      raise exceptions.ArgumentError(
          'Status of service [{}] is not ready'.format(serv_id))
    if tag:
      for t in serv.status.traffic:
        if t.tag == tag:
          if not t.url:
            raise exceptions.ArgumentError(
                'URL for tag [{}] in service [{}] is not ready'.format(
                    tag, serv_id))
          return t.url
      raise exceptions.ArgumentError(
          'Cannot find tag [{}] in service [{}].'.format(tag, serv_id))
    # If not tag provided, use the default service URL.
    if not serv.status.url:
      raise exceptions.ArgumentError(
          'URL for service [{}] is not ready'.format(serv_id))
    return serv.status.url


def _GetFreshIdToken():
  cred = store.LoadFreshCredential()
  credential = config_helper.Credential(cred)
  return credential.id_token



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command sub-group for Security Profiles Threat Prevention Profile."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class ThreatPrevention(base.Group):
  """Manage Security Profiles - Threat Prevention Profile.

  Manage Security Profiles - Threat Prevention Profile.

  ## EXAMPLES

  To create a Security Profile with the name `my-security-profile` which
  includes location as global or region specifed and organization ID, optional
  description as `New Security Profile`, run:

    $ {command} create my-security-profile  --description="New Security Profile"

  To add an override, run:

    $ {command} add-override my-security-profile --severities=MEDIUM
    --action=ALLOW

    `my-security-profile` is the name of the Security Profile in the
    format organizations/{organizationID}/locations/{location}/securityProfiles/
    {security_profile_id} where organizationID is the organization ID to which
    the changes should apply, location either global or region specified and
    security_profile_id the Security Profile Identifier.

  To update an override, run:

    $ {command} update-override my-security-profile --severities=MEDIUM
    --action=ALLOW

    `my-security-profile` is the name of the Security Profile in the
    format organizations/{organizationID}/locations/{location}/securityProfiles/
    {security_profile_id} where organizationID is the organization ID to which
    the changes should apply, location either global or region specified and
    security_profile_id the Security Profile Identifier.

  To list overrides, run:

    $ {command} list-overrides my-security-profile

    `my-security-profile` is the name of the Security Profile in the
    format organizations/{organizationID}/locations/{location}/securityProfiles/
    {security_profile_id} where organizationID is the organization ID to which
    the changes should apply, location either global or region specified and
    security_profile_id the Security Profile Identifier.

  To delete an override, run:

    $ {command} delete-override my-security-profile --severities=MEDIUM

    `my-security-profile` is the name of the Security Profile in the
    format organizations/{organizationID}/locations/{location}/securityProfiles/
    {security_profile_id} where organizationID is the organization ID to which
    the changes should apply, location either global or region specified and
    security_profile_id the Security Profile Identifier.

  To list Security Profiles in specifed location and organization, run:

    $ {command} list --location=global

  To delete a Security Profile called `my-security-profile` which includes
  location as global or region specifed and organization ID, run:

      $ {command} delete my-security-profile
  """
  category = base.NETWORK_SECURITY_CATEGORY
